@@ -8,7 +8,7 @@ use MRO::Compat;
 use Carp 'confess';
 use Scalar::Util 'blessed', 'weaken';
 
-our $VERSION   = '0.92_01';
+our $VERSION   = '0.97_01';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -28,15 +28,14 @@ sub superclasses {
 }
 
 sub _immutable_cannot_call {
-    Carp::confess "This method cannot be called on an immutable instance";
+    my $name = shift;
+    Carp::confess "The '$name' method cannot be called on an immutable instance";
 }
 
-sub add_method            { _immutable_cannot_call() }
-sub alias_method          { _immutable_cannot_call() }
-sub remove_method         { _immutable_cannot_call() }
-sub add_attribute         { _immutable_cannot_call() }
-sub remove_attribute      { _immutable_cannot_call() }
-sub remove_package_symbol { _immutable_cannot_call() }
+for my $name (qw/add_method alias_method remove_method add_attribute remove_attribute remove_package_symbol/) {
+    no strict 'refs';
+    *{__PACKAGE__."::$name"} = sub { _immutable_cannot_call($name) };
+}
 
 sub class_precedence_list {
     my $orig = shift;
@@ -75,10 +74,10 @@ sub get_meta_instance {
     $self->{__immutable}{get_meta_instance} ||= $self->$orig;
 }
 
-sub get_method_map {
+sub _get_method_map {
     my $orig = shift;
     my $self = shift;
-    $self->{__immutable}{get_method_map} ||= $self->$orig;
+    $self->{__immutable}{_get_method_map} ||= $self->$orig;
 }
 
 sub add_package_symbol {
